<?php

/**
 * Class FreinetPortalPluginIncomingRequestValidator
 * Validiert einen eingehenden Request darauf, dass die Datenfelder i.O. sind um evtl.
 * Injection-Angriffe zu verhindern und nur valide Werte für die Eingabeparameter
 * zuzulassen.
 */
class FreinetPortalPluginIncomingRequestValidator
{
    const ALLOWED_FREINET_PORTAL_REQUEST_PARAMS=array('umkreis',
											   'angebotplzs',
											   'zielgruppen',
											   'ziel2gruppen',
											   'kenntnisse',
											   'handlungsfelder',
											   'handlungs2felder',
											   'arbeitsweisen',
											   'globalbereiche',
											   'sprachen',
											   'sachspenden',
											   'zeitspenden',
											   'mtffa1',
											   'mtffa2',
											   'mtffa3',
											   'mtffa4',
											   'mtffa5',
											   'mtffa6',
											   'mtffa',
											   'portalmatrix1',
											   'portalmatrix2',
											   'ehrenamttyp',
											   'verknuepfen',
											   'suchbegriff',
											   'freinet_portal_result_page');

    const VALID_WP_ACTIONS=array('Suchen', 'angebot_anzeigen');
    const VALID_UMKREIS=array(0, 1, 2, 3);
    const FREINET_PORTAL_SEARCH_DATA='freinetSearchData';
    const LIST_OF_NUMBERS_PATTERN='/^\d+(,\d+)*$/';
    const ERROR_MESSAGE="Es scheinen unbekannte Daten im Request enthalten zu sein.";
    const VALID_SEARCH_CHARS='/^[a-zA-Z0-9\söÖüÜäÄß]+$/';
    private $request;

    /**
     * FreinetPortalPluginRequestValidator constructor.
     * @param $request
     */
    public function __construct($request)
    {
        $this->request=$request;
    }

    /**
     * @throws ErrorException wenn die Anfrage ungültige Daten enthält
     */
    public function validate()
    {
        if(
            $this->allRequestsParamAreKnown()
            &&$this->isFreinetPortalWpActionParameterValid()
            &&$this->isUmkreisAListedNumber()
            &&$this->isPlzValid()
            &&$this->isKriteriumValid('kenntnisse')
            &&$this->isKriteriumValid('handlungsfelder')
            &&$this->isKriteriumValid('handlungs2felder')
            &&$this->isKriteriumValid('arbeitsweisen')
            &&$this->isKriteriumValid('zielgruppen')
            &&$this->isKriteriumValid('ziel2gruppen')
            &&$this->isKriteriumValid('globalbereiche')
            &&$this->isKriteriumValid('zeitspenden')
            &&$this->isKriteriumValid('sachspenden')
            &&$this->isKriteriumValid('mtffa1')
            &&$this->isKriteriumValid('mtffa2')
            &&$this->isKriteriumValid('mtffa3')
            &&$this->isKriteriumValid('mtffa4')
            &&$this->isKriteriumValid('mtffa5')
            &&$this->isKriteriumValid('mtffa6')
            &&$this->isKriteriumValid('sprachen')
            &&$this->isVerknuepfenValid()
            &&$this->isSuchbegriffValid()
            &&$this->isPagingValid()
        )
        {
            return true;
        };
        throw new ErrorException(self::ERROR_MESSAGE);
    }

    private function isFreinetPortalWpActionParameterValid()
    {
        $action=$this->request['freinet_portal_wp_action'];
        if(empty($action)||in_array($action, self::VALID_WP_ACTIONS))
        {
            return true;
        }
        return false;
    }

    private function isUmkreisAListedNumber()
    {
        $umkreis=$this->fromRequest('umkreis');
        if(empty($umkreis)||preg_match('/^\d$/', $umkreis)&&in_array($umkreis, self::VALID_UMKREIS))
        {
            return true;
        };
        return false;
    }

    private function isPlzValid()
    {
        $angebotplzs=trim($this->fromRequest('angebotplzs'));
        if(empty($angebotplzs)||preg_match('/^\d{4,5}(,\d{4,5})*$/', $angebotplzs))
        {
            return true;
        }
        return false;
    }

    /**
     * @param $param string
     * @return bool
     */
    private function emptyOrListOfNumbers($param)
    {
        $value=$this->fromRequest($param);
        if(empty($value)||preg_match(self::LIST_OF_NUMBERS_PATTERN, $value))
        {
            return true;
        }
        return false;
    }

    private function isVerknuepfenValid()
    {
        $verknuepfen=$this->fromRequest('verknuepfen');
        if($verknuepfen=='verknuepfen'||empty($verknuepfen))
        {
            return true;
        }
        return false;
    }

    /**
     * @return mixed
     */
    private function fromRequest($value)
    {
        return $this->request[self::FREINET_PORTAL_SEARCH_DATA][$value];
    }

    private function allRequestsParamAreKnown()
    {

        $requestData=$this->request[self::FREINET_PORTAL_SEARCH_DATA];
        if(empty($requestData))
        {
            return true;
        }
        foreach($requestData as $key=>$ignore)
        {
            if(!in_array($key, self::ALLOWED_FREINET_PORTAL_REQUEST_PARAMS))
            {
                return false;
            }
        }
        return true;
    }

    private function isSuchbegriffValid()
    {
        $suchbegriff=$this->fromRequest('suchbegriff');
        if(empty($suchbegriff)||preg_match(self::VALID_SEARCH_CHARS, $suchbegriff))
        {
            return true;
        }
        return false;
    }

    private function isKriteriumValid($kriteriumValue)
    {
        $requestValue=$this->fromRequest($kriteriumValue);
        if(empty($requestValue))
        {
            return true;
        }
        if(is_string($requestValue))
        {
            return $this->emptyOrListOfNumbers($kriteriumValue);
        }
        if(is_array($requestValue))
        {
            return count(array_filter($requestValue, function ($value) {
                //return !preg_match('/^\d+$/', $value);
				return !preg_match(self::LIST_OF_NUMBERS_PATTERN, $value); // 2021-09-10 Zahlen UND Komma sind erlaubt
            }))==0 ? true : false;
        }
        return false;
    }

    private function isPagingValid()
    {
        $resultPage=$this->fromRequest('freinet_portal_result_page');
        if(empty($resultPage)||preg_match('/^\d+$/', $resultPage))
        {
            return true;
        }
        return false;
    }
}